@@ -1,7 +1,10 @@
 from dataclasses import dataclass
 from typing import List, Optional
-from serde import field, serialize, deserialize
+from serde import field, serialize, deserialize, to_dict
 from time import gmtime, strftime
+
+from trcli import settings
+
 from trcli.data_classes.validation_exception import ValidationException
 
 
@@ -19,6 +22,7 @@ class TestRailResult:
     defects: str = field(default=None, skip_if_default=True)
     assignedto_id: int = field(default=None, skip_if_default=True)
     attachments: Optional[List[str]] = field(default_factory=list, skip_if_default=True)
+    result_fields: Optional[dict] = field(default_factory=dict, skip=True)
     junit_result_unparsed: list = field(default=None, metadata={"serde_skip": True})
 
     def __post_init__(self):
@@ -32,6 +36,7 @@ class TestRailResult:
         if self.elapsed is not None:
             self.elapsed = self.proper_format_for_elapsed(self.elapsed)
 
+
     @staticmethod
     def calculate_status_id_from_junit_element(junit_result: list) -> int:
         """
@@ -63,12 +68,41 @@ class TestRailResult:
     @staticmethod
     def proper_format_for_elapsed(elapsed):
         try:
-            rounded_secs = round(float(elapsed))
-            return f"{rounded_secs}s" if rounded_secs > 0 else None
+            if settings.ALLOW_ELAPSED_MS:
+                return f"{round(elapsed, 3)}s" if float(elapsed) >= 0.001 else None
+            else:
+                elapsed = float(elapsed)
+                if elapsed > 1:
+                    return f"{round(elapsed)}s"
+                elif elapsed > 0:
+                    return "1s"
+                else:
+                    return None
         except ValueError:
             # unable to parse time format
-            return None
+            elapsed = None
 
+        return elapsed
+
+    def prepend_comment(self, comment: str):
+        self.comment = f"{comment}\n\n{self.comment}"
+
+    def add_global_result_fields(self, results_fields: dict) -> None:
+        """Add global result fields without overriding the existing test-specific result fields
+
+        :param results_fields: Global results fields to be added to the result
+        :return: None
+        """
+        if not results_fields:
+            return
+        new_results_fields = results_fields.copy()
+        new_results_fields.update(self.result_fields)
+        self.result_fields = new_results_fields
+
+    def to_dict(self) -> dict:
+        result_dict = to_dict(self)
+        result_dict.update(self.result_fields)
+        return result_dict
 
 @serialize
 @deserialize
@@ -84,6 +118,7 @@ class TestRailCase:
     type_id: int = field(default=None, skip_if_default=True)
     milestone_id: int = field(default=None, skip_if_default=True)
     refs: str = field(default=None, skip_if_default=True)
+    case_fields: Optional[dict] = field(default_factory=dict, skip=True)
     result: TestRailResult = field(default=None, metadata={"serde_skip": True})
     custom_automation_id: str = field(default=None, skip_if_default=True)
 
@@ -100,7 +135,25 @@ class TestRailCase:
                 class_name=self.__class__.__name__,
                 reason="Title is empty.",
             )
+        if self.custom_automation_id:
+            self.custom_automation_id = self.custom_automation_id.strip()
 
+    def add_global_case_fields(self, case_fields: dict) -> None:
+        """Add global case fields without overriding the existing case-specific fields
+
+        :param case_fields: Global case fields to be added to the result
+        :return: None
+        """
+        if not case_fields:
+            return
+        new_case_fields = case_fields.copy()
+        new_case_fields.update(self.case_fields)
+        self.case_fields = new_case_fields
+
+    def to_dict(self) -> dict:
+        case_dict = to_dict(self)
+        case_dict.update(self.case_fields)
+        return case_dict
 
 @serialize
 @deserialize
